@@ -3,8 +3,11 @@
 {-# LANGUAGE GADTs #-}
 #if __GLASGOW_HASKELL__ >= 707
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE Safe #-}
+#else
+-- Manual Typeable instances
+{-# LANGUAGE Trustworthy #-}
 #endif
-{-# OPTIONS_GHC -Wall #-}
 #include "free-common.h"
 
 -----------------------------------------------------------------------------
