@@ -7,8 +7,13 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE StandaloneDeriving #-}
+{-# LANGUAGE Safe #-}
+#else
+-- Manual Typeable instances
+{-# LANGUAGE Trustworthy #-}
 #endif
 #include "free-common.h"
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Monad.Free
@@ -52,9 +57,12 @@ import Control.Monad.Error.Class
 import Control.Monad.Cont.Class
 import Data.Functor.Bind
 import Data.Functor.Classes.Compat
+import Data.Functor.WithIndex
 import Data.Foldable
+import Data.Foldable.WithIndex
 import Data.Profunctor
 import Data.Traversable
+import Data.Traversable.WithIndex
 import Data.Semigroup.Foldable
 import Data.Semigroup.Traversable
 import Data.Data
@@ -63,6 +71,14 @@ import Prelude hiding (foldr)
 import GHC.Generics
 #endif
 
+-- $setup
+-- >>> import Control.Applicative (Const (..))
+-- >>> import Data.Functor.Identity (Identity (..))
+-- >>> import Data.Monoid (First (..))
+-- >>> import Data.Tagged (Tagged (..))
+-- >>> let preview l x = getFirst (getConst (l (Const . First . Just) x))
+-- >>> let review l x = runIdentity (unTagged (l (Tagged (Identity x))))
+
 -- | The 'Free' 'Monad' for a 'Functor' @f@.
 --
 -- /Formally/
@@ -295,6 +311,21 @@ instance Traversable1 f => Traversable1 (Free f) where
     go (Free fa) = Free <$> traverse1 go fa
   {-# INLINE traverse1 #-}
 
+instance FunctorWithIndex i f => FunctorWithIndex [i] (Free f) where
+  imap f (Pure a) = Pure $ f [] a
+  imap f (Free s) = Free $ imap (\i -> imap (f . (:) i)) s
+  {-# INLINE imap #-}
+
+instance FoldableWithIndex i f => FoldableWithIndex [i] (Free f) where
+  ifoldMap f (Pure a) = f [] a
+  ifoldMap f (Free s) = ifoldMap (\i -> ifoldMap (f . (:) i)) s
+  {-# INLINE ifoldMap #-}
+
+instance TraversableWithIndex i f => TraversableWithIndex [i] (Free f) where
+  itraverse f (Pure a) = Pure <$> f [] a
+  itraverse f (Free s) = Free <$> itraverse (\i -> itraverse (f . (:) i)) s
+  {-# INLINE itraverse #-}
+
 instance (Functor m, MonadWriter e m) => MonadWriter e (Free m) where
   tell = lift . tell
   {-# INLINE tell #-}
